// Copyright (c) 2019, Christoph Peters
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Karlsruhe Institute of Technology nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#pragma once
#include <cmath>
#include "complex_algebra.hpp"


void trigonometricToExponentialMomentsReal3(float_complex pOutExponentialMoment[3],const float pTrigonometricMoment[3]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal4(float_complex pOutExponentialMoment[4],const float pTrigonometricMoment[4]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal5(float_complex pOutExponentialMoment[5],const float pTrigonometricMoment[5]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[4]=pTrigonometricMoment[4]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[3]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]+pTrigonometricMoment[2]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]+pTrigonometricMoment[1]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal6(float_complex pOutExponentialMoment[6],const float pTrigonometricMoment[6]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[4]=pTrigonometricMoment[4]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[3]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]+pTrigonometricMoment[2]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]+pTrigonometricMoment[1]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3];
	pOutExponentialMoment[5]=pTrigonometricMoment[5]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[4]*float_complex(0.0f,5.02654825f)*pOutExponentialMoment[1]+pTrigonometricMoment[3]*float_complex(0.0f,3.76991118f)*pOutExponentialMoment[2]+pTrigonometricMoment[2]*float_complex(0.0f,2.51327412f)*pOutExponentialMoment[3]+pTrigonometricMoment[1]*float_complex(0.0f,1.25663706f)*pOutExponentialMoment[4];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal7(float_complex pOutExponentialMoment[7],const float pTrigonometricMoment[7]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[4]=pTrigonometricMoment[4]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[3]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]+pTrigonometricMoment[2]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]+pTrigonometricMoment[1]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3];
	pOutExponentialMoment[5]=pTrigonometricMoment[5]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[4]*float_complex(0.0f,5.02654825f)*pOutExponentialMoment[1]+pTrigonometricMoment[3]*float_complex(0.0f,3.76991118f)*pOutExponentialMoment[2]+pTrigonometricMoment[2]*float_complex(0.0f,2.51327412f)*pOutExponentialMoment[3]+pTrigonometricMoment[1]*float_complex(0.0f,1.25663706f)*pOutExponentialMoment[4];
	pOutExponentialMoment[6]=pTrigonometricMoment[6]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[5]*float_complex(0.0f,5.23598776f)*pOutExponentialMoment[1]+pTrigonometricMoment[4]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[2]+pTrigonometricMoment[3]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[3]+pTrigonometricMoment[2]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[4]+pTrigonometricMoment[1]*float_complex(0.0f,1.04719755f)*pOutExponentialMoment[5];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal8(float_complex pOutExponentialMoment[8],const float pTrigonometricMoment[8]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[4]=pTrigonometricMoment[4]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[3]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]+pTrigonometricMoment[2]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]+pTrigonometricMoment[1]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3];
	pOutExponentialMoment[5]=pTrigonometricMoment[5]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[4]*float_complex(0.0f,5.02654825f)*pOutExponentialMoment[1]+pTrigonometricMoment[3]*float_complex(0.0f,3.76991118f)*pOutExponentialMoment[2]+pTrigonometricMoment[2]*float_complex(0.0f,2.51327412f)*pOutExponentialMoment[3]+pTrigonometricMoment[1]*float_complex(0.0f,1.25663706f)*pOutExponentialMoment[4];
	pOutExponentialMoment[6]=pTrigonometricMoment[6]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[5]*float_complex(0.0f,5.23598776f)*pOutExponentialMoment[1]+pTrigonometricMoment[4]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[2]+pTrigonometricMoment[3]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[3]+pTrigonometricMoment[2]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[4]+pTrigonometricMoment[1]*float_complex(0.0f,1.04719755f)*pOutExponentialMoment[5];
	pOutExponentialMoment[7]=pTrigonometricMoment[7]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[6]*float_complex(0.0f,5.38558741f)*pOutExponentialMoment[1]+pTrigonometricMoment[5]*float_complex(0.0f,4.48798951f)*pOutExponentialMoment[2]+pTrigonometricMoment[4]*float_complex(0.0f,3.5903916f)*pOutExponentialMoment[3]+pTrigonometricMoment[3]*float_complex(0.0f,2.6927937f)*pOutExponentialMoment[4]+pTrigonometricMoment[2]*float_complex(0.0f,1.7951958f)*pOutExponentialMoment[5]+pTrigonometricMoment[1]*float_complex(0.0f,0.897597901f)*pOutExponentialMoment[6];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMomentsReal9(float_complex pOutExponentialMoment[9],const float pTrigonometricMoment[9]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0]-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=pTrigonometricMoment[1]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0];
	pOutExponentialMoment[2]=pTrigonometricMoment[2]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[1]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1];
	pOutExponentialMoment[3]=pTrigonometricMoment[3]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[2]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]+pTrigonometricMoment[1]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2];
	pOutExponentialMoment[4]=pTrigonometricMoment[4]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[3]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]+pTrigonometricMoment[2]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]+pTrigonometricMoment[1]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3];
	pOutExponentialMoment[5]=pTrigonometricMoment[5]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[4]*float_complex(0.0f,5.02654825f)*pOutExponentialMoment[1]+pTrigonometricMoment[3]*float_complex(0.0f,3.76991118f)*pOutExponentialMoment[2]+pTrigonometricMoment[2]*float_complex(0.0f,2.51327412f)*pOutExponentialMoment[3]+pTrigonometricMoment[1]*float_complex(0.0f,1.25663706f)*pOutExponentialMoment[4];
	pOutExponentialMoment[6]=pTrigonometricMoment[6]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[5]*float_complex(0.0f,5.23598776f)*pOutExponentialMoment[1]+pTrigonometricMoment[4]*float_complex(0.0f,4.1887902f)*pOutExponentialMoment[2]+pTrigonometricMoment[3]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[3]+pTrigonometricMoment[2]*float_complex(0.0f,2.0943951f)*pOutExponentialMoment[4]+pTrigonometricMoment[1]*float_complex(0.0f,1.04719755f)*pOutExponentialMoment[5];
	pOutExponentialMoment[7]=pTrigonometricMoment[7]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[6]*float_complex(0.0f,5.38558741f)*pOutExponentialMoment[1]+pTrigonometricMoment[5]*float_complex(0.0f,4.48798951f)*pOutExponentialMoment[2]+pTrigonometricMoment[4]*float_complex(0.0f,3.5903916f)*pOutExponentialMoment[3]+pTrigonometricMoment[3]*float_complex(0.0f,2.6927937f)*pOutExponentialMoment[4]+pTrigonometricMoment[2]*float_complex(0.0f,1.7951958f)*pOutExponentialMoment[5]+pTrigonometricMoment[1]*float_complex(0.0f,0.897597901f)*pOutExponentialMoment[6];
	pOutExponentialMoment[8]=pTrigonometricMoment[8]*float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]+pTrigonometricMoment[7]*float_complex(0.0f,5.49778714f)*pOutExponentialMoment[1]+pTrigonometricMoment[6]*float_complex(0.0f,4.71238898f)*pOutExponentialMoment[2]+pTrigonometricMoment[5]*float_complex(0.0f,3.92699082f)*pOutExponentialMoment[3]+pTrigonometricMoment[4]*float_complex(0.0f,3.14159265f)*pOutExponentialMoment[4]+pTrigonometricMoment[3]*float_complex(0.0f,2.35619449f)*pOutExponentialMoment[5]+pTrigonometricMoment[2]*float_complex(0.0f,1.57079633f)*pOutExponentialMoment[6]+pTrigonometricMoment[1]*float_complex(0.0f,0.785398163f)*pOutExponentialMoment[7];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMoments2(float_complex pOutExponentialMoment[2],const float_complex pTrigonometricMoment[2]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0].x-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[1];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMoments3(float_complex pOutExponentialMoment[3],const float_complex pTrigonometricMoment[3]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0].x-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[1];
	pOutExponentialMoment[2]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[2]+float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1]*pTrigonometricMoment[1];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMoments4(float_complex pOutExponentialMoment[4],const float_complex pTrigonometricMoment[4]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0].x-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[1];
	pOutExponentialMoment[2]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[2]+float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1]*pTrigonometricMoment[1];
	pOutExponentialMoment[3]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[3]+float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]*pTrigonometricMoment[2]+float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2]*pTrigonometricMoment[1];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}

void trigonometricToExponentialMoments5(float_complex pOutExponentialMoment[5],const float_complex pTrigonometricMoment[5]){
	float zerothMomentPhase=3.14159265f*pTrigonometricMoment[0].x-1.57079633f;
	pOutExponentialMoment[0]=float_complex(cosf(zerothMomentPhase),sinf(zerothMomentPhase));
	pOutExponentialMoment[0]=0.0795774715f*pOutExponentialMoment[0];
	pOutExponentialMoment[1]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[1];
	pOutExponentialMoment[2]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[2]+float_complex(0.0f,3.14159265f)*pOutExponentialMoment[1]*pTrigonometricMoment[1];
	pOutExponentialMoment[3]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[3]+float_complex(0.0f,4.1887902f)*pOutExponentialMoment[1]*pTrigonometricMoment[2]+float_complex(0.0f,2.0943951f)*pOutExponentialMoment[2]*pTrigonometricMoment[1];
	pOutExponentialMoment[4]=float_complex(0.0f,6.28318531f)*pOutExponentialMoment[0]*pTrigonometricMoment[4]+float_complex(0.0f,4.71238898f)*pOutExponentialMoment[1]*pTrigonometricMoment[3]+float_complex(0.0f,3.14159265f)*pOutExponentialMoment[2]*pTrigonometricMoment[2]+float_complex(0.0f,1.57079633f)*pOutExponentialMoment[3]*pTrigonometricMoment[1];
	pOutExponentialMoment[0]=2.0f*pOutExponentialMoment[0];
}


void levinsonsAlgorithm2(float_complex pOutSolution[2],const float_complex pFirstColumn[2]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing2(float_complex pOutSolution[2],float_complex pFirstColumn[2]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
}

void levinsonsAlgorithm3(float_complex pOutSolution[3],const float_complex pFirstColumn[3]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing3(float_complex pOutSolution[3],float_complex pFirstColumn[3]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
}

void levinsonsAlgorithm4(float_complex pOutSolution[4],const float_complex pFirstColumn[4]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing4(float_complex pOutSolution[4],float_complex pFirstColumn[4]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
}

void levinsonsAlgorithm5(float_complex pOutSolution[5],const float_complex pFirstColumn[5]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing5(float_complex pOutSolution[5],float_complex pFirstColumn[5]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[4]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
}

void levinsonsAlgorithm6(float_complex pOutSolution[6],const float_complex pFirstColumn[6]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing6(float_complex pOutSolution[6],float_complex pFirstColumn[6]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[4]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[5]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
}

void levinsonsAlgorithm7(float_complex pOutSolution[7],const float_complex pFirstColumn[7]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing7(float_complex pOutSolution[7],float_complex pFirstColumn[7]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[4]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[5]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[6]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
}

void levinsonsAlgorithm8(float_complex pOutSolution[8],const float_complex pFirstColumn[8]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float_complex flippedSolution7;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[6]+pOutSolution[2]*pFirstColumn[5]+pOutSolution[3]*pFirstColumn[4]+pOutSolution[4]*pFirstColumn[3]+pOutSolution[5]*pFirstColumn[2]+pOutSolution[6]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[7]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[6]);
	flippedSolution2=conjugate(pOutSolution[5]);
	flippedSolution3=conjugate(pOutSolution[4]);
	flippedSolution4=conjugate(pOutSolution[3]);
	flippedSolution5=conjugate(pOutSolution[2]);
	flippedSolution6=conjugate(pOutSolution[1]);
	flippedSolution7=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(-flippedSolution7.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing8(float_complex pOutSolution[8],float_complex pFirstColumn[8]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float_complex flippedSolution7;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[4]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[5]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[6]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[6]+pOutSolution[2]*pFirstColumn[5]+pOutSolution[3]*pFirstColumn[4]+pOutSolution[4]*pFirstColumn[3]+pOutSolution[5]*pFirstColumn[2]+pOutSolution[6]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[7]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[7]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[6]);
	flippedSolution2=conjugate(pOutSolution[5]);
	flippedSolution3=conjugate(pOutSolution[4]);
	flippedSolution4=conjugate(pOutSolution[3]);
	flippedSolution5=conjugate(pOutSolution[2]);
	flippedSolution6=conjugate(pOutSolution[1]);
	flippedSolution7=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(-flippedSolution7.x*dotProduct);
}

void levinsonsAlgorithm9(float_complex pOutSolution[9],const float_complex pFirstColumn[9]){
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float_complex flippedSolution7;
	float_complex flippedSolution8;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[6]+pOutSolution[2]*pFirstColumn[5]+pOutSolution[3]*pFirstColumn[4]+pOutSolution[4]*pFirstColumn[3]+pOutSolution[5]*pFirstColumn[2]+pOutSolution[6]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[7]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[6]);
	flippedSolution2=conjugate(pOutSolution[5]);
	flippedSolution3=conjugate(pOutSolution[4]);
	flippedSolution4=conjugate(pOutSolution[3]);
	flippedSolution5=conjugate(pOutSolution[2]);
	flippedSolution6=conjugate(pOutSolution[1]);
	flippedSolution7=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(-flippedSolution7.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[7]+pOutSolution[2]*pFirstColumn[6]+pOutSolution[3]*pFirstColumn[5]+pOutSolution[4]*pFirstColumn[4]+pOutSolution[5]*pFirstColumn[3]+pOutSolution[6]*pFirstColumn[2]+pOutSolution[7]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[8]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	flippedSolution1=conjugate(pOutSolution[7]);
	flippedSolution2=conjugate(pOutSolution[6]);
	flippedSolution3=conjugate(pOutSolution[5]);
	flippedSolution4=conjugate(pOutSolution[4]);
	flippedSolution5=conjugate(pOutSolution[3]);
	flippedSolution6=conjugate(pOutSolution[2]);
	flippedSolution7=conjugate(pOutSolution[1]);
	flippedSolution8=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(pOutSolution[7]-flippedSolution7*dotProduct);
	pOutSolution[8]=factor*(-flippedSolution8.x*dotProduct);
}

void levinsonsAlgorithmWithBiasing9(float_complex pOutSolution[9],float_complex pFirstColumn[9]){
	float oneMinusCorrectionBias=0.9999f;
	float correctedFactor=1.0f/(1.0f-oneMinusCorrectionBias*oneMinusCorrectionBias);
	pOutSolution[0]=float_complex(1.0f/(pFirstColumn[0].x),0.0f);
	float_complex unnormalizedNextMomentCenter;
	float_complex dotProduct;
	float dotProductAbsSquare;
	float_complex flippedSolution1;
	float_complex flippedSolution2;
	float_complex flippedSolution3;
	float_complex flippedSolution4;
	float_complex flippedSolution5;
	float_complex flippedSolution6;
	float_complex flippedSolution7;
	float_complex flippedSolution8;
	float factor;
	unnormalizedNextMomentCenter=float_complex(0.0f,0.0f);
	dotProduct=pOutSolution[0].x*pFirstColumn[1]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[1]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(-flippedSolution1.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[2]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[2]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[1]);
	flippedSolution2=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(-flippedSolution2.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[2]+pOutSolution[2]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[3]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[3]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[2]);
	flippedSolution2=conjugate(pOutSolution[1]);
	flippedSolution3=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(-flippedSolution3.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[3]+pOutSolution[2]*pFirstColumn[2]+pOutSolution[3]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[4]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[4]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[3]);
	flippedSolution2=conjugate(pOutSolution[2]);
	flippedSolution3=conjugate(pOutSolution[1]);
	flippedSolution4=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(-flippedSolution4.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[4]+pOutSolution[2]*pFirstColumn[3]+pOutSolution[3]*pFirstColumn[2]+pOutSolution[4]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[5]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[5]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[4]);
	flippedSolution2=conjugate(pOutSolution[3]);
	flippedSolution3=conjugate(pOutSolution[2]);
	flippedSolution4=conjugate(pOutSolution[1]);
	flippedSolution5=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(-flippedSolution5.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[5]+pOutSolution[2]*pFirstColumn[4]+pOutSolution[3]*pFirstColumn[3]+pOutSolution[4]*pFirstColumn[2]+pOutSolution[5]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[6]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[6]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[5]);
	flippedSolution2=conjugate(pOutSolution[4]);
	flippedSolution3=conjugate(pOutSolution[3]);
	flippedSolution4=conjugate(pOutSolution[2]);
	flippedSolution5=conjugate(pOutSolution[1]);
	flippedSolution6=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(-flippedSolution6.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[6]+pOutSolution[2]*pFirstColumn[5]+pOutSolution[3]*pFirstColumn[4]+pOutSolution[4]*pFirstColumn[3]+pOutSolution[5]*pFirstColumn[2]+pOutSolution[6]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[7]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[7]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
		oneMinusCorrectionBias=0.0f;
		correctedFactor=1.0f;
	}
	
	flippedSolution1=conjugate(pOutSolution[6]);
	flippedSolution2=conjugate(pOutSolution[5]);
	flippedSolution3=conjugate(pOutSolution[4]);
	flippedSolution4=conjugate(pOutSolution[3]);
	flippedSolution5=conjugate(pOutSolution[2]);
	flippedSolution6=conjugate(pOutSolution[1]);
	flippedSolution7=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(-flippedSolution7.x*dotProduct);
	unnormalizedNextMomentCenter=pOutSolution[1]*pFirstColumn[7]+pOutSolution[2]*pFirstColumn[6]+pOutSolution[3]*pFirstColumn[5]+pOutSolution[4]*pFirstColumn[4]+pOutSolution[5]*pFirstColumn[3]+pOutSolution[6]*pFirstColumn[2]+pOutSolution[7]*pFirstColumn[1];
	dotProduct=pOutSolution[0].x*pFirstColumn[8]+unnormalizedNextMomentCenter;
	dotProductAbsSquare=absSqr(dotProduct);
	factor=1.0f/(1.0f-dotProductAbsSquare);
	if(factor<0.0f){
		dotProduct=(oneMinusCorrectionBias*1.0f/std::sqrtf(dotProductAbsSquare))*dotProduct;
		pFirstColumn[8]=(dotProduct-unnormalizedNextMomentCenter)/(pOutSolution[0].x);
		factor=correctedFactor;
	}
	
	flippedSolution1=conjugate(pOutSolution[7]);
	flippedSolution2=conjugate(pOutSolution[6]);
	flippedSolution3=conjugate(pOutSolution[5]);
	flippedSolution4=conjugate(pOutSolution[4]);
	flippedSolution5=conjugate(pOutSolution[3]);
	flippedSolution6=conjugate(pOutSolution[2]);
	flippedSolution7=conjugate(pOutSolution[1]);
	flippedSolution8=float_complex(pOutSolution[0].x,0.0f);
	pOutSolution[0]=float_complex(factor*pOutSolution[0].x,0.0f);
	pOutSolution[1]=factor*(pOutSolution[1]-flippedSolution1*dotProduct);
	pOutSolution[2]=factor*(pOutSolution[2]-flippedSolution2*dotProduct);
	pOutSolution[3]=factor*(pOutSolution[3]-flippedSolution3*dotProduct);
	pOutSolution[4]=factor*(pOutSolution[4]-flippedSolution4*dotProduct);
	pOutSolution[5]=factor*(pOutSolution[5]-flippedSolution5*dotProduct);
	pOutSolution[6]=factor*(pOutSolution[6]-flippedSolution6*dotProduct);
	pOutSolution[7]=factor*(pOutSolution[7]-flippedSolution7*dotProduct);
	pOutSolution[8]=factor*(-flippedSolution8.x*dotProduct);
}


void computeAutocorrelation2(float_complex pOutAutocorrelation[2],const float_complex pSignal[2]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1]);
}

void computeAutocorrelation3(float_complex pOutAutocorrelation[3],const float_complex pSignal[3]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2]);
}

void computeAutocorrelation4(float_complex pOutAutocorrelation[4],const float_complex pSignal[4]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3]);
}

void computeAutocorrelation5(float_complex pOutAutocorrelation[5],const float_complex pSignal[5]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3])+pSignal[4]*conjugate(pSignal[4]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3])+pSignal[3]*conjugate(pSignal[4]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3])+pSignal[2]*conjugate(pSignal[4]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3])+pSignal[1]*conjugate(pSignal[4]);
	pOutAutocorrelation[4]=pSignal[0]*conjugate(pSignal[4]);
}

void computeAutocorrelation6(float_complex pOutAutocorrelation[6],const float_complex pSignal[6]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3])+pSignal[4]*conjugate(pSignal[4])+pSignal[5]*conjugate(pSignal[5]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3])+pSignal[3]*conjugate(pSignal[4])+pSignal[4]*conjugate(pSignal[5]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3])+pSignal[2]*conjugate(pSignal[4])+pSignal[3]*conjugate(pSignal[5]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3])+pSignal[1]*conjugate(pSignal[4])+pSignal[2]*conjugate(pSignal[5]);
	pOutAutocorrelation[4]=pSignal[0]*conjugate(pSignal[4])+pSignal[1]*conjugate(pSignal[5]);
	pOutAutocorrelation[5]=pSignal[0]*conjugate(pSignal[5]);
}

void computeAutocorrelation7(float_complex pOutAutocorrelation[7],const float_complex pSignal[7]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3])+pSignal[4]*conjugate(pSignal[4])+pSignal[5]*conjugate(pSignal[5])+pSignal[6]*conjugate(pSignal[6]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3])+pSignal[3]*conjugate(pSignal[4])+pSignal[4]*conjugate(pSignal[5])+pSignal[5]*conjugate(pSignal[6]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3])+pSignal[2]*conjugate(pSignal[4])+pSignal[3]*conjugate(pSignal[5])+pSignal[4]*conjugate(pSignal[6]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3])+pSignal[1]*conjugate(pSignal[4])+pSignal[2]*conjugate(pSignal[5])+pSignal[3]*conjugate(pSignal[6]);
	pOutAutocorrelation[4]=pSignal[0]*conjugate(pSignal[4])+pSignal[1]*conjugate(pSignal[5])+pSignal[2]*conjugate(pSignal[6]);
	pOutAutocorrelation[5]=pSignal[0]*conjugate(pSignal[5])+pSignal[1]*conjugate(pSignal[6]);
	pOutAutocorrelation[6]=pSignal[0]*conjugate(pSignal[6]);
}

void computeAutocorrelation8(float_complex pOutAutocorrelation[8],const float_complex pSignal[8]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3])+pSignal[4]*conjugate(pSignal[4])+pSignal[5]*conjugate(pSignal[5])+pSignal[6]*conjugate(pSignal[6])+pSignal[7]*conjugate(pSignal[7]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3])+pSignal[3]*conjugate(pSignal[4])+pSignal[4]*conjugate(pSignal[5])+pSignal[5]*conjugate(pSignal[6])+pSignal[6]*conjugate(pSignal[7]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3])+pSignal[2]*conjugate(pSignal[4])+pSignal[3]*conjugate(pSignal[5])+pSignal[4]*conjugate(pSignal[6])+pSignal[5]*conjugate(pSignal[7]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3])+pSignal[1]*conjugate(pSignal[4])+pSignal[2]*conjugate(pSignal[5])+pSignal[3]*conjugate(pSignal[6])+pSignal[4]*conjugate(pSignal[7]);
	pOutAutocorrelation[4]=pSignal[0]*conjugate(pSignal[4])+pSignal[1]*conjugate(pSignal[5])+pSignal[2]*conjugate(pSignal[6])+pSignal[3]*conjugate(pSignal[7]);
	pOutAutocorrelation[5]=pSignal[0]*conjugate(pSignal[5])+pSignal[1]*conjugate(pSignal[6])+pSignal[2]*conjugate(pSignal[7]);
	pOutAutocorrelation[6]=pSignal[0]*conjugate(pSignal[6])+pSignal[1]*conjugate(pSignal[7]);
	pOutAutocorrelation[7]=pSignal[0]*conjugate(pSignal[7]);
}

void computeAutocorrelation9(float_complex pOutAutocorrelation[9],const float_complex pSignal[9]){
	pOutAutocorrelation[0]=pSignal[0]*conjugate(pSignal[0])+pSignal[1]*conjugate(pSignal[1])+pSignal[2]*conjugate(pSignal[2])+pSignal[3]*conjugate(pSignal[3])+pSignal[4]*conjugate(pSignal[4])+pSignal[5]*conjugate(pSignal[5])+pSignal[6]*conjugate(pSignal[6])+pSignal[7]*conjugate(pSignal[7])+pSignal[8]*conjugate(pSignal[8]);
	pOutAutocorrelation[1]=pSignal[0]*conjugate(pSignal[1])+pSignal[1]*conjugate(pSignal[2])+pSignal[2]*conjugate(pSignal[3])+pSignal[3]*conjugate(pSignal[4])+pSignal[4]*conjugate(pSignal[5])+pSignal[5]*conjugate(pSignal[6])+pSignal[6]*conjugate(pSignal[7])+pSignal[7]*conjugate(pSignal[8]);
	pOutAutocorrelation[2]=pSignal[0]*conjugate(pSignal[2])+pSignal[1]*conjugate(pSignal[3])+pSignal[2]*conjugate(pSignal[4])+pSignal[3]*conjugate(pSignal[5])+pSignal[4]*conjugate(pSignal[6])+pSignal[5]*conjugate(pSignal[7])+pSignal[6]*conjugate(pSignal[8]);
	pOutAutocorrelation[3]=pSignal[0]*conjugate(pSignal[3])+pSignal[1]*conjugate(pSignal[4])+pSignal[2]*conjugate(pSignal[5])+pSignal[3]*conjugate(pSignal[6])+pSignal[4]*conjugate(pSignal[7])+pSignal[5]*conjugate(pSignal[8]);
	pOutAutocorrelation[4]=pSignal[0]*conjugate(pSignal[4])+pSignal[1]*conjugate(pSignal[5])+pSignal[2]*conjugate(pSignal[6])+pSignal[3]*conjugate(pSignal[7])+pSignal[4]*conjugate(pSignal[8]);
	pOutAutocorrelation[5]=pSignal[0]*conjugate(pSignal[5])+pSignal[1]*conjugate(pSignal[6])+pSignal[2]*conjugate(pSignal[7])+pSignal[3]*conjugate(pSignal[8]);
	pOutAutocorrelation[6]=pSignal[0]*conjugate(pSignal[6])+pSignal[1]*conjugate(pSignal[7])+pSignal[2]*conjugate(pSignal[8]);
	pOutAutocorrelation[7]=pSignal[0]*conjugate(pSignal[7])+pSignal[1]*conjugate(pSignal[8]);
	pOutAutocorrelation[8]=pSignal[0]*conjugate(pSignal[8]);
}


void computeCorrelation2(float_complex pOutCorrelation[2],const float_complex pLHS[2],const float_complex pRHS[2]){
	pOutCorrelation[0]=pLHS[0]*pRHS[0]+pLHS[1]*pRHS[1];
	pOutCorrelation[1]=pLHS[1]*pRHS[0];
}

void computeCorrelation3(float_complex pOutCorrelation[3],const float_complex pLHS[3],const float_complex pRHS[3]){
	pOutCorrelation[0]=pLHS[0]*pRHS[0]+pLHS[1]*pRHS[1]+pLHS[2]*pRHS[2];
	pOutCorrelation[1]=pLHS[1]*pRHS[0]+pLHS[2]*pRHS[1];
	pOutCorrelation[2]=pLHS[2]*pRHS[0];
}

void computeCorrelation4(float_complex pOutCorrelation[4],const float_complex pLHS[4],const float_complex pRHS[4]){
	pOutCorrelation[0]=pLHS[0]*pRHS[0]+pLHS[1]*pRHS[1]+pLHS[2]*pRHS[2]+pLHS[3]*pRHS[3];
	pOutCorrelation[1]=pLHS[1]*pRHS[0]+pLHS[2]*pRHS[1]+pLHS[3]*pRHS[2];
	pOutCorrelation[2]=pLHS[2]*pRHS[0]+pLHS[3]*pRHS[1];
	pOutCorrelation[3]=pLHS[3]*pRHS[0];
}

void computeCorrelation5(float_complex pOutCorrelation[5],const float_complex pLHS[5],const float_complex pRHS[5]){
	pOutCorrelation[0]=pLHS[0]*pRHS[0]+pLHS[1]*pRHS[1]+pLHS[2]*pRHS[2]+pLHS[3]*pRHS[3]+pLHS[4]*pRHS[4];
	pOutCorrelation[1]=pLHS[1]*pRHS[0]+pLHS[2]*pRHS[1]+pLHS[3]*pRHS[2]+pLHS[4]*pRHS[3];
	pOutCorrelation[2]=pLHS[2]*pRHS[0]+pLHS[3]*pRHS[1]+pLHS[4]*pRHS[2];
	pOutCorrelation[3]=pLHS[3]*pRHS[0]+pLHS[4]*pRHS[1];
	pOutCorrelation[4]=pLHS[4]*pRHS[0];
}


void computeImaginaryCorrelation3(float pOutCorrelation[3],const float_complex pLHS[3],const float_complex pRHS[3]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x;
}

void computeImaginaryCorrelation4(float pOutCorrelation[4],const float_complex pLHS[4],const float_complex pRHS[4]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x;
}

void computeImaginaryCorrelation5(float pOutCorrelation[5],const float_complex pLHS[5],const float_complex pRHS[5]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x+pLHS[4].x*pRHS[4].y+pLHS[4].y*pRHS[4].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x+pLHS[4].x*pRHS[3].y+pLHS[4].y*pRHS[3].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x+pLHS[4].x*pRHS[2].y+pLHS[4].y*pRHS[2].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x+pLHS[4].x*pRHS[1].y+pLHS[4].y*pRHS[1].x;
	pOutCorrelation[4]=pLHS[4].x*pRHS[0].y+pLHS[4].y*pRHS[0].x;
}

void computeImaginaryCorrelation6(float pOutCorrelation[6],const float_complex pLHS[6],const float_complex pRHS[6]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x+pLHS[4].x*pRHS[4].y+pLHS[4].y*pRHS[4].x+pLHS[5].x*pRHS[5].y+pLHS[5].y*pRHS[5].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x+pLHS[4].x*pRHS[3].y+pLHS[4].y*pRHS[3].x+pLHS[5].x*pRHS[4].y+pLHS[5].y*pRHS[4].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x+pLHS[4].x*pRHS[2].y+pLHS[4].y*pRHS[2].x+pLHS[5].x*pRHS[3].y+pLHS[5].y*pRHS[3].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x+pLHS[4].x*pRHS[1].y+pLHS[4].y*pRHS[1].x+pLHS[5].x*pRHS[2].y+pLHS[5].y*pRHS[2].x;
	pOutCorrelation[4]=pLHS[4].x*pRHS[0].y+pLHS[4].y*pRHS[0].x+pLHS[5].x*pRHS[1].y+pLHS[5].y*pRHS[1].x;
	pOutCorrelation[5]=pLHS[5].x*pRHS[0].y+pLHS[5].y*pRHS[0].x;
}

void computeImaginaryCorrelation7(float pOutCorrelation[7],const float_complex pLHS[7],const float_complex pRHS[7]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x+pLHS[4].x*pRHS[4].y+pLHS[4].y*pRHS[4].x+pLHS[5].x*pRHS[5].y+pLHS[5].y*pRHS[5].x+pLHS[6].x*pRHS[6].y+pLHS[6].y*pRHS[6].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x+pLHS[4].x*pRHS[3].y+pLHS[4].y*pRHS[3].x+pLHS[5].x*pRHS[4].y+pLHS[5].y*pRHS[4].x+pLHS[6].x*pRHS[5].y+pLHS[6].y*pRHS[5].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x+pLHS[4].x*pRHS[2].y+pLHS[4].y*pRHS[2].x+pLHS[5].x*pRHS[3].y+pLHS[5].y*pRHS[3].x+pLHS[6].x*pRHS[4].y+pLHS[6].y*pRHS[4].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x+pLHS[4].x*pRHS[1].y+pLHS[4].y*pRHS[1].x+pLHS[5].x*pRHS[2].y+pLHS[5].y*pRHS[2].x+pLHS[6].x*pRHS[3].y+pLHS[6].y*pRHS[3].x;
	pOutCorrelation[4]=pLHS[4].x*pRHS[0].y+pLHS[4].y*pRHS[0].x+pLHS[5].x*pRHS[1].y+pLHS[5].y*pRHS[1].x+pLHS[6].x*pRHS[2].y+pLHS[6].y*pRHS[2].x;
	pOutCorrelation[5]=pLHS[5].x*pRHS[0].y+pLHS[5].y*pRHS[0].x+pLHS[6].x*pRHS[1].y+pLHS[6].y*pRHS[1].x;
	pOutCorrelation[6]=pLHS[6].x*pRHS[0].y+pLHS[6].y*pRHS[0].x;
}

void computeImaginaryCorrelation8(float pOutCorrelation[8],const float_complex pLHS[8],const float_complex pRHS[8]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x+pLHS[4].x*pRHS[4].y+pLHS[4].y*pRHS[4].x+pLHS[5].x*pRHS[5].y+pLHS[5].y*pRHS[5].x+pLHS[6].x*pRHS[6].y+pLHS[6].y*pRHS[6].x+pLHS[7].x*pRHS[7].y+pLHS[7].y*pRHS[7].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x+pLHS[4].x*pRHS[3].y+pLHS[4].y*pRHS[3].x+pLHS[5].x*pRHS[4].y+pLHS[5].y*pRHS[4].x+pLHS[6].x*pRHS[5].y+pLHS[6].y*pRHS[5].x+pLHS[7].x*pRHS[6].y+pLHS[7].y*pRHS[6].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x+pLHS[4].x*pRHS[2].y+pLHS[4].y*pRHS[2].x+pLHS[5].x*pRHS[3].y+pLHS[5].y*pRHS[3].x+pLHS[6].x*pRHS[4].y+pLHS[6].y*pRHS[4].x+pLHS[7].x*pRHS[5].y+pLHS[7].y*pRHS[5].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x+pLHS[4].x*pRHS[1].y+pLHS[4].y*pRHS[1].x+pLHS[5].x*pRHS[2].y+pLHS[5].y*pRHS[2].x+pLHS[6].x*pRHS[3].y+pLHS[6].y*pRHS[3].x+pLHS[7].x*pRHS[4].y+pLHS[7].y*pRHS[4].x;
	pOutCorrelation[4]=pLHS[4].x*pRHS[0].y+pLHS[4].y*pRHS[0].x+pLHS[5].x*pRHS[1].y+pLHS[5].y*pRHS[1].x+pLHS[6].x*pRHS[2].y+pLHS[6].y*pRHS[2].x+pLHS[7].x*pRHS[3].y+pLHS[7].y*pRHS[3].x;
	pOutCorrelation[5]=pLHS[5].x*pRHS[0].y+pLHS[5].y*pRHS[0].x+pLHS[6].x*pRHS[1].y+pLHS[6].y*pRHS[1].x+pLHS[7].x*pRHS[2].y+pLHS[7].y*pRHS[2].x;
	pOutCorrelation[6]=pLHS[6].x*pRHS[0].y+pLHS[6].y*pRHS[0].x+pLHS[7].x*pRHS[1].y+pLHS[7].y*pRHS[1].x;
	pOutCorrelation[7]=pLHS[7].x*pRHS[0].y+pLHS[7].y*pRHS[0].x;
}

void computeImaginaryCorrelation9(float pOutCorrelation[9],const float_complex pLHS[9],const float_complex pRHS[9]){
	pOutCorrelation[0]=pLHS[0].x*pRHS[0].y+pLHS[0].y*pRHS[0].x+pLHS[1].x*pRHS[1].y+pLHS[1].y*pRHS[1].x+pLHS[2].x*pRHS[2].y+pLHS[2].y*pRHS[2].x+pLHS[3].x*pRHS[3].y+pLHS[3].y*pRHS[3].x+pLHS[4].x*pRHS[4].y+pLHS[4].y*pRHS[4].x+pLHS[5].x*pRHS[5].y+pLHS[5].y*pRHS[5].x+pLHS[6].x*pRHS[6].y+pLHS[6].y*pRHS[6].x+pLHS[7].x*pRHS[7].y+pLHS[7].y*pRHS[7].x+pLHS[8].x*pRHS[8].y+pLHS[8].y*pRHS[8].x;
	pOutCorrelation[1]=pLHS[1].x*pRHS[0].y+pLHS[1].y*pRHS[0].x+pLHS[2].x*pRHS[1].y+pLHS[2].y*pRHS[1].x+pLHS[3].x*pRHS[2].y+pLHS[3].y*pRHS[2].x+pLHS[4].x*pRHS[3].y+pLHS[4].y*pRHS[3].x+pLHS[5].x*pRHS[4].y+pLHS[5].y*pRHS[4].x+pLHS[6].x*pRHS[5].y+pLHS[6].y*pRHS[5].x+pLHS[7].x*pRHS[6].y+pLHS[7].y*pRHS[6].x+pLHS[8].x*pRHS[7].y+pLHS[8].y*pRHS[7].x;
	pOutCorrelation[2]=pLHS[2].x*pRHS[0].y+pLHS[2].y*pRHS[0].x+pLHS[3].x*pRHS[1].y+pLHS[3].y*pRHS[1].x+pLHS[4].x*pRHS[2].y+pLHS[4].y*pRHS[2].x+pLHS[5].x*pRHS[3].y+pLHS[5].y*pRHS[3].x+pLHS[6].x*pRHS[4].y+pLHS[6].y*pRHS[4].x+pLHS[7].x*pRHS[5].y+pLHS[7].y*pRHS[5].x+pLHS[8].x*pRHS[6].y+pLHS[8].y*pRHS[6].x;
	pOutCorrelation[3]=pLHS[3].x*pRHS[0].y+pLHS[3].y*pRHS[0].x+pLHS[4].x*pRHS[1].y+pLHS[4].y*pRHS[1].x+pLHS[5].x*pRHS[2].y+pLHS[5].y*pRHS[2].x+pLHS[6].x*pRHS[3].y+pLHS[6].y*pRHS[3].x+pLHS[7].x*pRHS[4].y+pLHS[7].y*pRHS[4].x+pLHS[8].x*pRHS[5].y+pLHS[8].y*pRHS[5].x;
	pOutCorrelation[4]=pLHS[4].x*pRHS[0].y+pLHS[4].y*pRHS[0].x+pLHS[5].x*pRHS[1].y+pLHS[5].y*pRHS[1].x+pLHS[6].x*pRHS[2].y+pLHS[6].y*pRHS[2].x+pLHS[7].x*pRHS[3].y+pLHS[7].y*pRHS[3].x+pLHS[8].x*pRHS[4].y+pLHS[8].y*pRHS[4].x;
	pOutCorrelation[5]=pLHS[5].x*pRHS[0].y+pLHS[5].y*pRHS[0].x+pLHS[6].x*pRHS[1].y+pLHS[6].y*pRHS[1].x+pLHS[7].x*pRHS[2].y+pLHS[7].y*pRHS[2].x+pLHS[8].x*pRHS[3].y+pLHS[8].y*pRHS[3].x;
	pOutCorrelation[6]=pLHS[6].x*pRHS[0].y+pLHS[6].y*pRHS[0].x+pLHS[7].x*pRHS[1].y+pLHS[7].y*pRHS[1].x+pLHS[8].x*pRHS[2].y+pLHS[8].y*pRHS[2].x;
	pOutCorrelation[7]=pLHS[7].x*pRHS[0].y+pLHS[7].y*pRHS[0].x+pLHS[8].x*pRHS[1].y+pLHS[8].y*pRHS[1].x;
	pOutCorrelation[8]=pLHS[8].x*pRHS[0].y+pLHS[8].y*pRHS[0].x;
}


float_complex evaluateFastHerglotzTransform2(const float_complex circlePoint,const float_complex pExponentialMoment[2],const float_complex pEvaluationPolynomial[2]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial1=pEvaluationPolynomial[0].x;
	float_complex polynomial0=pEvaluationPolynomial[1]+polynomial1*conjCirclePoint;
	float_complex dotProduct=polynomial1*pExponentialMoment[1];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform3(const float_complex circlePoint,const float_complex pExponentialMoment[3],const float_complex pEvaluationPolynomial[3]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial2=pEvaluationPolynomial[0].x;
	float_complex polynomial1=pEvaluationPolynomial[1]+polynomial2*conjCirclePoint;
	float_complex polynomial0=pEvaluationPolynomial[2]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform4(const float_complex circlePoint,const float_complex pExponentialMoment[4],const float_complex pEvaluationPolynomial[4]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial3=pEvaluationPolynomial[0].x;
	float_complex polynomial2=pEvaluationPolynomial[1]+polynomial3*conjCirclePoint;
	float_complex polynomial1=pEvaluationPolynomial[2]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[3]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform5(const float_complex circlePoint,const float_complex pExponentialMoment[5],const float_complex pEvaluationPolynomial[5]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial4=pEvaluationPolynomial[0].x;
	float_complex polynomial3=pEvaluationPolynomial[1]+polynomial4*conjCirclePoint;
	float_complex polynomial2=pEvaluationPolynomial[2]+conjCirclePoint*polynomial3;
	float_complex polynomial1=pEvaluationPolynomial[3]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[4]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3]+polynomial4*pExponentialMoment[4];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform6(const float_complex circlePoint,const float_complex pExponentialMoment[6],const float_complex pEvaluationPolynomial[6]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial5=pEvaluationPolynomial[0].x;
	float_complex polynomial4=pEvaluationPolynomial[1]+polynomial5*conjCirclePoint;
	float_complex polynomial3=pEvaluationPolynomial[2]+conjCirclePoint*polynomial4;
	float_complex polynomial2=pEvaluationPolynomial[3]+conjCirclePoint*polynomial3;
	float_complex polynomial1=pEvaluationPolynomial[4]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[5]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3]+polynomial4*pExponentialMoment[4]+polynomial5*pExponentialMoment[5];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform7(const float_complex circlePoint,const float_complex pExponentialMoment[7],const float_complex pEvaluationPolynomial[7]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial6=pEvaluationPolynomial[0].x;
	float_complex polynomial5=pEvaluationPolynomial[1]+polynomial6*conjCirclePoint;
	float_complex polynomial4=pEvaluationPolynomial[2]+conjCirclePoint*polynomial5;
	float_complex polynomial3=pEvaluationPolynomial[3]+conjCirclePoint*polynomial4;
	float_complex polynomial2=pEvaluationPolynomial[4]+conjCirclePoint*polynomial3;
	float_complex polynomial1=pEvaluationPolynomial[5]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[6]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3]+polynomial4*pExponentialMoment[4]+polynomial5*pExponentialMoment[5]+polynomial6*pExponentialMoment[6];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform8(const float_complex circlePoint,const float_complex pExponentialMoment[8],const float_complex pEvaluationPolynomial[8]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial7=pEvaluationPolynomial[0].x;
	float_complex polynomial6=pEvaluationPolynomial[1]+polynomial7*conjCirclePoint;
	float_complex polynomial5=pEvaluationPolynomial[2]+conjCirclePoint*polynomial6;
	float_complex polynomial4=pEvaluationPolynomial[3]+conjCirclePoint*polynomial5;
	float_complex polynomial3=pEvaluationPolynomial[4]+conjCirclePoint*polynomial4;
	float_complex polynomial2=pEvaluationPolynomial[5]+conjCirclePoint*polynomial3;
	float_complex polynomial1=pEvaluationPolynomial[6]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[7]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3]+polynomial4*pExponentialMoment[4]+polynomial5*pExponentialMoment[5]+polynomial6*pExponentialMoment[6]+polynomial7*pExponentialMoment[7];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}

float_complex evaluateFastHerglotzTransform9(const float_complex circlePoint,const float_complex pExponentialMoment[9],const float_complex pEvaluationPolynomial[9]){
	float_complex conjCirclePoint=conjugate(circlePoint);
	float polynomial8=pEvaluationPolynomial[0].x;
	float_complex polynomial7=pEvaluationPolynomial[1]+polynomial8*conjCirclePoint;
	float_complex polynomial6=pEvaluationPolynomial[2]+conjCirclePoint*polynomial7;
	float_complex polynomial5=pEvaluationPolynomial[3]+conjCirclePoint*polynomial6;
	float_complex polynomial4=pEvaluationPolynomial[4]+conjCirclePoint*polynomial5;
	float_complex polynomial3=pEvaluationPolynomial[5]+conjCirclePoint*polynomial4;
	float_complex polynomial2=pEvaluationPolynomial[6]+conjCirclePoint*polynomial3;
	float_complex polynomial1=pEvaluationPolynomial[7]+conjCirclePoint*polynomial2;
	float_complex polynomial0=pEvaluationPolynomial[8]+conjCirclePoint*polynomial1;
	float_complex dotProduct=polynomial1*pExponentialMoment[1]+polynomial2*pExponentialMoment[2]+polynomial3*pExponentialMoment[3]+polynomial4*pExponentialMoment[4]+polynomial5*pExponentialMoment[5]+polynomial6*pExponentialMoment[6]+polynomial7*pExponentialMoment[7]+polynomial8*pExponentialMoment[8];
	return pExponentialMoment[0]+2.0f*(dotProduct)/(polynomial0);
}


float evaluateFourierSeries2(const float_complex circlePoint,const float_complex pFourierCoefficient[2]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float result=pFourierCoefficient[1].x*circlePointPower1.x-pFourierCoefficient[1].y*circlePointPower1.y;
	return 2.0f*result+pFourierCoefficient[0].x;
}

float evaluateFourierSeries3(const float_complex circlePoint,const float_complex pFourierCoefficient[3]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float result=pFourierCoefficient[1].x*circlePointPower1.x-pFourierCoefficient[1].y*circlePointPower1.y+pFourierCoefficient[2].x*circlePointPower2.x-pFourierCoefficient[2].y*circlePointPower2.y;
	return 2.0f*result+pFourierCoefficient[0].x;
}

float evaluateFourierSeries4(const float_complex circlePoint,const float_complex pFourierCoefficient[4]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float result=pFourierCoefficient[1].x*circlePointPower1.x-pFourierCoefficient[1].y*circlePointPower1.y+pFourierCoefficient[2].x*circlePointPower2.x-pFourierCoefficient[2].y*circlePointPower2.y+pFourierCoefficient[3].x*circlePointPower3.x-pFourierCoefficient[3].y*circlePointPower3.y;
	return 2.0f*result+pFourierCoefficient[0].x;
}

float evaluateFourierSeries5(const float_complex circlePoint,const float_complex pFourierCoefficient[5]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float result=pFourierCoefficient[1].x*circlePointPower1.x-pFourierCoefficient[1].y*circlePointPower1.y+pFourierCoefficient[2].x*circlePointPower2.x-pFourierCoefficient[2].y*circlePointPower2.y+pFourierCoefficient[3].x*circlePointPower3.x-pFourierCoefficient[3].y*circlePointPower3.y+pFourierCoefficient[4].x*circlePointPower4.x-pFourierCoefficient[4].y*circlePointPower4.y;
	return 2.0f*result+pFourierCoefficient[0].x;
}


float evaluateFourierSeriesReal3(const float_complex circlePoint,const float pFourierCoefficient[3]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal4(const float_complex circlePoint,const float pFourierCoefficient[4]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal5(const float_complex circlePoint,const float pFourierCoefficient[5]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x+pFourierCoefficient[4]*circlePointPower4.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal6(const float_complex circlePoint,const float pFourierCoefficient[6]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float_complex circlePointPower5=circlePointPower1*circlePointPower4;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x+pFourierCoefficient[4]*circlePointPower4.x+pFourierCoefficient[5]*circlePointPower5.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal7(const float_complex circlePoint,const float pFourierCoefficient[7]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float_complex circlePointPower5=circlePointPower1*circlePointPower4;
	float_complex circlePointPower6=circlePointPower2*circlePointPower4;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x+pFourierCoefficient[4]*circlePointPower4.x+pFourierCoefficient[5]*circlePointPower5.x+pFourierCoefficient[6]*circlePointPower6.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal8(const float_complex circlePoint,const float pFourierCoefficient[8]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float_complex circlePointPower5=circlePointPower1*circlePointPower4;
	float_complex circlePointPower6=circlePointPower2*circlePointPower4;
	float_complex circlePointPower7=circlePointPower3*circlePointPower4;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x+pFourierCoefficient[4]*circlePointPower4.x+pFourierCoefficient[5]*circlePointPower5.x+pFourierCoefficient[6]*circlePointPower6.x+pFourierCoefficient[7]*circlePointPower7.x;
	return 2.0f*result+pFourierCoefficient[0];
}

float evaluateFourierSeriesReal9(const float_complex circlePoint,const float pFourierCoefficient[9]){
	// Minimize sequential dependencies by computing powers of circlePoint
	// by multiplying previous powers by powers of two
	float_complex circlePointPower1=circlePoint;
	float_complex circlePointPower2=circlePointPower1*circlePointPower1;
	float_complex circlePointPower3=circlePointPower1*circlePointPower2;
	float_complex circlePointPower4=circlePointPower2*circlePointPower2;
	float_complex circlePointPower5=circlePointPower1*circlePointPower4;
	float_complex circlePointPower6=circlePointPower2*circlePointPower4;
	float_complex circlePointPower7=circlePointPower3*circlePointPower4;
	float_complex circlePointPower8=circlePointPower4*circlePointPower4;
	float result=pFourierCoefficient[1]*circlePointPower1.x+pFourierCoefficient[2]*circlePointPower2.x+pFourierCoefficient[3]*circlePointPower3.x+pFourierCoefficient[4]*circlePointPower4.x+pFourierCoefficient[5]*circlePointPower5.x+pFourierCoefficient[6]*circlePointPower6.x+pFourierCoefficient[7]*circlePointPower7.x+pFourierCoefficient[8]*circlePointPower8.x;
	return 2.0f*result+pFourierCoefficient[0];
}


void prepareReflectanceSpectrum2(float_complex pOutExponentialMoment[2],float_complex pOutEvaluationPolynomial[2],const float_complex pTrigonometricMoment[2]){
	trigonometricToExponentialMoments2(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm2(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
}

void prepareReflectanceSpectrumWithBiasing2(float_complex pOutExponentialMoment[2],float_complex pOutEvaluationPolynomial[2],float_complex pTrigonometricMoment[2]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	trigonometricToExponentialMoments2(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing2(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
}

void prepareReflectanceSpectrum3(float_complex pOutExponentialMoment[3],float_complex pOutEvaluationPolynomial[3],const float_complex pTrigonometricMoment[3]){
	trigonometricToExponentialMoments3(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm3(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
}

void prepareReflectanceSpectrumWithBiasing3(float_complex pOutExponentialMoment[3],float_complex pOutEvaluationPolynomial[3],float_complex pTrigonometricMoment[3]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	trigonometricToExponentialMoments3(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing3(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
}

void prepareReflectanceSpectrum4(float_complex pOutExponentialMoment[4],float_complex pOutEvaluationPolynomial[4],const float_complex pTrigonometricMoment[4]){
	trigonometricToExponentialMoments4(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm4(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
}

void prepareReflectanceSpectrumWithBiasing4(float_complex pOutExponentialMoment[4],float_complex pOutEvaluationPolynomial[4],float_complex pTrigonometricMoment[4]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	trigonometricToExponentialMoments4(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing4(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
}

void prepareReflectanceSpectrum5(float_complex pOutExponentialMoment[5],float_complex pOutEvaluationPolynomial[5],const float_complex pTrigonometricMoment[5]){
	trigonometricToExponentialMoments5(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm5(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
}

void prepareReflectanceSpectrumWithBiasing5(float_complex pOutExponentialMoment[5],float_complex pOutEvaluationPolynomial[5],float_complex pTrigonometricMoment[5]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	trigonometricToExponentialMoments5(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing5(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
}

void prepareReflectanceSpectrumReal3(float_complex pOutExponentialMoment[3],float_complex pOutEvaluationPolynomial[3],const float pTrigonometricMoment[3]){
	trigonometricToExponentialMomentsReal3(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm3(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
}

void prepareReflectanceSpectrumRealWithBiasing3(float_complex pOutExponentialMoment[3],float_complex pOutEvaluationPolynomial[3],float pTrigonometricMoment[3]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal3(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing3(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
}

void prepareReflectanceSpectrumReal4(float_complex pOutExponentialMoment[4],float_complex pOutEvaluationPolynomial[4],const float pTrigonometricMoment[4]){
	trigonometricToExponentialMomentsReal4(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm4(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
}

void prepareReflectanceSpectrumRealWithBiasing4(float_complex pOutExponentialMoment[4],float_complex pOutEvaluationPolynomial[4],float pTrigonometricMoment[4]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal4(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing4(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
}

void prepareReflectanceSpectrumReal5(float_complex pOutExponentialMoment[5],float_complex pOutEvaluationPolynomial[5],const float pTrigonometricMoment[5]){
	trigonometricToExponentialMomentsReal5(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm5(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
}

void prepareReflectanceSpectrumRealWithBiasing5(float_complex pOutExponentialMoment[5],float_complex pOutEvaluationPolynomial[5],float pTrigonometricMoment[5]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal5(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing5(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
}

void prepareReflectanceSpectrumReal6(float_complex pOutExponentialMoment[6],float_complex pOutEvaluationPolynomial[6],const float pTrigonometricMoment[6]){
	trigonometricToExponentialMomentsReal6(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm6(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
}

void prepareReflectanceSpectrumRealWithBiasing6(float_complex pOutExponentialMoment[6],float_complex pOutEvaluationPolynomial[6],float pTrigonometricMoment[6]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal6(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing6(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
}

void prepareReflectanceSpectrumReal7(float_complex pOutExponentialMoment[7],float_complex pOutEvaluationPolynomial[7],const float pTrigonometricMoment[7]){
	trigonometricToExponentialMomentsReal7(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm7(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
}

void prepareReflectanceSpectrumRealWithBiasing7(float_complex pOutExponentialMoment[7],float_complex pOutEvaluationPolynomial[7],float pTrigonometricMoment[7]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal7(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing7(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
}

void prepareReflectanceSpectrumReal8(float_complex pOutExponentialMoment[8],float_complex pOutEvaluationPolynomial[8],const float pTrigonometricMoment[8]){
	trigonometricToExponentialMomentsReal8(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm8(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
	pOutEvaluationPolynomial[7]=6.28318531f*pOutEvaluationPolynomial[7];
}

void prepareReflectanceSpectrumRealWithBiasing8(float_complex pOutExponentialMoment[8],float_complex pOutEvaluationPolynomial[8],float pTrigonometricMoment[8]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal8(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing8(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
	pOutEvaluationPolynomial[7]=6.28318531f*pOutEvaluationPolynomial[7];
}

void prepareReflectanceSpectrumReal9(float_complex pOutExponentialMoment[9],float_complex pOutEvaluationPolynomial[9],const float pTrigonometricMoment[9]){
	trigonometricToExponentialMomentsReal9(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithm9(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
	pOutEvaluationPolynomial[7]=6.28318531f*pOutEvaluationPolynomial[7];
	pOutEvaluationPolynomial[8]=6.28318531f*pOutEvaluationPolynomial[8];
}

void prepareReflectanceSpectrumRealWithBiasing9(float_complex pOutExponentialMoment[9],float_complex pOutEvaluationPolynomial[9],float pTrigonometricMoment[9]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	trigonometricToExponentialMomentsReal9(pOutExponentialMoment,pTrigonometricMoment);
	levinsonsAlgorithmWithBiasing9(pOutEvaluationPolynomial,pOutExponentialMoment);
	pOutEvaluationPolynomial[0]=6.28318531f*pOutEvaluationPolynomial[0];
	pOutEvaluationPolynomial[1]=6.28318531f*pOutEvaluationPolynomial[1];
	pOutEvaluationPolynomial[2]=6.28318531f*pOutEvaluationPolynomial[2];
	pOutEvaluationPolynomial[3]=6.28318531f*pOutEvaluationPolynomial[3];
	pOutEvaluationPolynomial[4]=6.28318531f*pOutEvaluationPolynomial[4];
	pOutEvaluationPolynomial[5]=6.28318531f*pOutEvaluationPolynomial[5];
	pOutEvaluationPolynomial[6]=6.28318531f*pOutEvaluationPolynomial[6];
	pOutEvaluationPolynomial[7]=6.28318531f*pOutEvaluationPolynomial[7];
	pOutEvaluationPolynomial[8]=6.28318531f*pOutEvaluationPolynomial[8];
}


float evaluateReflectanceSpectrum2(const float phase,const float_complex pExponentialMoment[2],const float_complex pEvaluationPolynomial[2]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform2(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum3(const float phase,const float_complex pExponentialMoment[3],const float_complex pEvaluationPolynomial[3]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform3(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum4(const float phase,const float_complex pExponentialMoment[4],const float_complex pEvaluationPolynomial[4]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform4(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum5(const float phase,const float_complex pExponentialMoment[5],const float_complex pEvaluationPolynomial[5]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform5(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum6(const float phase,const float_complex pExponentialMoment[6],const float_complex pEvaluationPolynomial[6]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform6(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum7(const float phase,const float_complex pExponentialMoment[7],const float_complex pEvaluationPolynomial[7]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform7(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum8(const float phase,const float_complex pExponentialMoment[8],const float_complex pEvaluationPolynomial[8]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform8(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrum9(const float phase,const float_complex pExponentialMoment[9],const float_complex pEvaluationPolynomial[9]){
	float_complex circlePoint;
	circlePoint=float_complex(cosf(phase),sinf(phase));
	float_complex herglotzTransform;
	herglotzTransform=evaluateFastHerglotzTransform9(circlePoint,pExponentialMoment,pEvaluationPolynomial);
	return fast_atan(herglotzTransform.y/herglotzTransform.x)*0.318309886f+0.5f;
}


void prepareReflectanceSpectrumLagrange2(float_complex pOutLagrangeMultiplier[2],const float_complex pTrigonometricMoment[2]){
	float_complex pExponentialMoment[2];
	trigonometricToExponentialMoments2(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[2];
	levinsonsAlgorithm2(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	float_complex pAutocorrelation[2];
	computeAutocorrelation2(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation2(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrangeWithBiasing2(float_complex pOutLagrangeMultiplier[2],float_complex pTrigonometricMoment[2]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	float_complex pExponentialMoment[2];
	trigonometricToExponentialMoments2(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[2];
	levinsonsAlgorithmWithBiasing2(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	float_complex pAutocorrelation[2];
	computeAutocorrelation2(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation2(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrange3(float_complex pOutLagrangeMultiplier[3],const float_complex pTrigonometricMoment[3]){
	float_complex pExponentialMoment[3];
	trigonometricToExponentialMoments3(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[3];
	levinsonsAlgorithm3(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	float_complex pAutocorrelation[3];
	computeAutocorrelation3(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation3(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrangeWithBiasing3(float_complex pOutLagrangeMultiplier[3],float_complex pTrigonometricMoment[3]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	float_complex pExponentialMoment[3];
	trigonometricToExponentialMoments3(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[3];
	levinsonsAlgorithmWithBiasing3(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	float_complex pAutocorrelation[3];
	computeAutocorrelation3(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation3(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrange4(float_complex pOutLagrangeMultiplier[4],const float_complex pTrigonometricMoment[4]){
	float_complex pExponentialMoment[4];
	trigonometricToExponentialMoments4(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[4];
	levinsonsAlgorithm4(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	float_complex pAutocorrelation[4];
	computeAutocorrelation4(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation4(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[3]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[3]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrangeWithBiasing4(float_complex pOutLagrangeMultiplier[4],float_complex pTrigonometricMoment[4]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	float_complex pExponentialMoment[4];
	trigonometricToExponentialMoments4(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[4];
	levinsonsAlgorithmWithBiasing4(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	float_complex pAutocorrelation[4];
	computeAutocorrelation4(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation4(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[3]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[3]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrange5(float_complex pOutLagrangeMultiplier[5],const float_complex pTrigonometricMoment[5]){
	float_complex pExponentialMoment[5];
	trigonometricToExponentialMoments5(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[5];
	levinsonsAlgorithm5(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	float_complex pAutocorrelation[5];
	computeAutocorrelation5(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation5(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[3]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[3]);
	pOutLagrangeMultiplier[4]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[4]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrangeWithBiasing5(float_complex pOutLagrangeMultiplier[5],float_complex pTrigonometricMoment[5]){
	pTrigonometricMoment[0]=float_complex((pTrigonometricMoment[0].x<0.0001f)?0.0001f:((pTrigonometricMoment[0].x>0.9999f)?0.9999f:pTrigonometricMoment[0].x),0.0f);
	float_complex pExponentialMoment[5];
	trigonometricToExponentialMoments5(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[5];
	levinsonsAlgorithmWithBiasing5(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	float_complex pAutocorrelation[5];
	computeAutocorrelation5(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeCorrelation5(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[0]);
	pOutLagrangeMultiplier[1]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[1]);
	pOutLagrangeMultiplier[2]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[2]);
	pOutLagrangeMultiplier[3]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[3]);
	pOutLagrangeMultiplier[4]=normalizationFactor*(float_complex(-0.0f,-1.0f)*pOutLagrangeMultiplier[4]);
	pOutLagrangeMultiplier[0]=float_complex(pOutLagrangeMultiplier[0].x,0.0f);
}

void prepareReflectanceSpectrumLagrangeReal3(float pOutLagrangeMultiplier[3],const float pTrigonometricMoment[3]){
	float_complex pExponentialMoment[3];
	trigonometricToExponentialMomentsReal3(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[3];
	levinsonsAlgorithm3(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	float_complex pAutocorrelation[3];
	computeAutocorrelation3(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation3(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing3(float pOutLagrangeMultiplier[3],float pTrigonometricMoment[3]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[3];
	trigonometricToExponentialMomentsReal3(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[3];
	levinsonsAlgorithmWithBiasing3(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	float_complex pAutocorrelation[3];
	computeAutocorrelation3(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation3(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
}

void prepareReflectanceSpectrumLagrangeReal4(float pOutLagrangeMultiplier[4],const float pTrigonometricMoment[4]){
	float_complex pExponentialMoment[4];
	trigonometricToExponentialMomentsReal4(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[4];
	levinsonsAlgorithm4(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	float_complex pAutocorrelation[4];
	computeAutocorrelation4(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation4(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing4(float pOutLagrangeMultiplier[4],float pTrigonometricMoment[4]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[4];
	trigonometricToExponentialMomentsReal4(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[4];
	levinsonsAlgorithmWithBiasing4(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	float_complex pAutocorrelation[4];
	computeAutocorrelation4(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation4(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
}

void prepareReflectanceSpectrumLagrangeReal5(float pOutLagrangeMultiplier[5],const float pTrigonometricMoment[5]){
	float_complex pExponentialMoment[5];
	trigonometricToExponentialMomentsReal5(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[5];
	levinsonsAlgorithm5(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	float_complex pAutocorrelation[5];
	computeAutocorrelation5(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation5(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing5(float pOutLagrangeMultiplier[5],float pTrigonometricMoment[5]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[5];
	trigonometricToExponentialMomentsReal5(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[5];
	levinsonsAlgorithmWithBiasing5(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	float_complex pAutocorrelation[5];
	computeAutocorrelation5(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation5(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
}

void prepareReflectanceSpectrumLagrangeReal6(float pOutLagrangeMultiplier[6],const float pTrigonometricMoment[6]){
	float_complex pExponentialMoment[6];
	trigonometricToExponentialMomentsReal6(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[6];
	levinsonsAlgorithm6(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	float_complex pAutocorrelation[6];
	computeAutocorrelation6(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation6(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing6(float pOutLagrangeMultiplier[6],float pTrigonometricMoment[6]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[6];
	trigonometricToExponentialMomentsReal6(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[6];
	levinsonsAlgorithmWithBiasing6(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	float_complex pAutocorrelation[6];
	computeAutocorrelation6(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation6(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
}

void prepareReflectanceSpectrumLagrangeReal7(float pOutLagrangeMultiplier[7],const float pTrigonometricMoment[7]){
	float_complex pExponentialMoment[7];
	trigonometricToExponentialMomentsReal7(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[7];
	levinsonsAlgorithm7(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	float_complex pAutocorrelation[7];
	computeAutocorrelation7(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation7(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing7(float pOutLagrangeMultiplier[7],float pTrigonometricMoment[7]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[7];
	trigonometricToExponentialMomentsReal7(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[7];
	levinsonsAlgorithmWithBiasing7(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	float_complex pAutocorrelation[7];
	computeAutocorrelation7(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation7(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
}

void prepareReflectanceSpectrumLagrangeReal8(float pOutLagrangeMultiplier[8],const float pTrigonometricMoment[8]){
	float_complex pExponentialMoment[8];
	trigonometricToExponentialMomentsReal8(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[8];
	levinsonsAlgorithm8(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	pEvaluationPolynomial[7]=6.28318531f*pEvaluationPolynomial[7];
	float_complex pAutocorrelation[8];
	computeAutocorrelation8(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation8(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
	pOutLagrangeMultiplier[7]=normalizationFactor*pOutLagrangeMultiplier[7];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing8(float pOutLagrangeMultiplier[8],float pTrigonometricMoment[8]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[8];
	trigonometricToExponentialMomentsReal8(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[8];
	levinsonsAlgorithmWithBiasing8(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	pEvaluationPolynomial[7]=6.28318531f*pEvaluationPolynomial[7];
	float_complex pAutocorrelation[8];
	computeAutocorrelation8(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation8(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
	pOutLagrangeMultiplier[7]=normalizationFactor*pOutLagrangeMultiplier[7];
}

void prepareReflectanceSpectrumLagrangeReal9(float pOutLagrangeMultiplier[9],const float pTrigonometricMoment[9]){
	float_complex pExponentialMoment[9];
	trigonometricToExponentialMomentsReal9(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[9];
	levinsonsAlgorithm9(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	pEvaluationPolynomial[7]=6.28318531f*pEvaluationPolynomial[7];
	pEvaluationPolynomial[8]=6.28318531f*pEvaluationPolynomial[8];
	float_complex pAutocorrelation[9];
	computeAutocorrelation9(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation9(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
	pOutLagrangeMultiplier[7]=normalizationFactor*pOutLagrangeMultiplier[7];
	pOutLagrangeMultiplier[8]=normalizationFactor*pOutLagrangeMultiplier[8];
}

void prepareReflectanceSpectrumLagrangeRealWithBiasing9(float pOutLagrangeMultiplier[9],float pTrigonometricMoment[9]){
	pTrigonometricMoment[0]=(pTrigonometricMoment[0]<0.0001f)?0.0001f:((pTrigonometricMoment[0]>0.9999f)?0.9999f:pTrigonometricMoment[0]);
	float_complex pExponentialMoment[9];
	trigonometricToExponentialMomentsReal9(pExponentialMoment,pTrigonometricMoment);
	float_complex pEvaluationPolynomial[9];
	levinsonsAlgorithmWithBiasing9(pEvaluationPolynomial,pExponentialMoment);
	pEvaluationPolynomial[0]=6.28318531f*pEvaluationPolynomial[0];
	pEvaluationPolynomial[1]=6.28318531f*pEvaluationPolynomial[1];
	pEvaluationPolynomial[2]=6.28318531f*pEvaluationPolynomial[2];
	pEvaluationPolynomial[3]=6.28318531f*pEvaluationPolynomial[3];
	pEvaluationPolynomial[4]=6.28318531f*pEvaluationPolynomial[4];
	pEvaluationPolynomial[5]=6.28318531f*pEvaluationPolynomial[5];
	pEvaluationPolynomial[6]=6.28318531f*pEvaluationPolynomial[6];
	pEvaluationPolynomial[7]=6.28318531f*pEvaluationPolynomial[7];
	pEvaluationPolynomial[8]=6.28318531f*pEvaluationPolynomial[8];
	float_complex pAutocorrelation[9];
	computeAutocorrelation9(pAutocorrelation,pEvaluationPolynomial);
	pExponentialMoment[0]=0.5f*pExponentialMoment[0];
	computeImaginaryCorrelation9(pOutLagrangeMultiplier,pAutocorrelation,pExponentialMoment);
	float normalizationFactor=1.0f/(3.14159265f*pEvaluationPolynomial[0].x);
	pOutLagrangeMultiplier[0]=normalizationFactor*pOutLagrangeMultiplier[0];
	pOutLagrangeMultiplier[1]=normalizationFactor*pOutLagrangeMultiplier[1];
	pOutLagrangeMultiplier[2]=normalizationFactor*pOutLagrangeMultiplier[2];
	pOutLagrangeMultiplier[3]=normalizationFactor*pOutLagrangeMultiplier[3];
	pOutLagrangeMultiplier[4]=normalizationFactor*pOutLagrangeMultiplier[4];
	pOutLagrangeMultiplier[5]=normalizationFactor*pOutLagrangeMultiplier[5];
	pOutLagrangeMultiplier[6]=normalizationFactor*pOutLagrangeMultiplier[6];
	pOutLagrangeMultiplier[7]=normalizationFactor*pOutLagrangeMultiplier[7];
	pOutLagrangeMultiplier[8]=normalizationFactor*pOutLagrangeMultiplier[8];
}


float evaluateReflectanceSpectrumLagrange2(const float phase,const float_complex pLagrangeMultiplier[2]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeries2(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrange3(const float phase,const float_complex pLagrangeMultiplier[3]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeries3(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrange4(const float phase,const float_complex pLagrangeMultiplier[4]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeries4(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrange5(const float phase,const float_complex pLagrangeMultiplier[5]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeries5(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal3(const float phase,const float pLagrangeMultiplier[3]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal3(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal4(const float phase,const float pLagrangeMultiplier[4]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal4(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal5(const float phase,const float pLagrangeMultiplier[5]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal5(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal6(const float phase,const float pLagrangeMultiplier[6]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal6(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal7(const float phase,const float pLagrangeMultiplier[7]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal7(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal8(const float phase,const float pLagrangeMultiplier[8]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal8(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}

float evaluateReflectanceSpectrumLagrangeReal9(const float phase,const float pLagrangeMultiplier[9]){
	float_complex conjCirclePoint;
	conjCirclePoint=float_complex(cosf(-phase),sinf(-phase));
	float lagrangeSeries;
	lagrangeSeries=evaluateFourierSeriesReal9(conjCirclePoint,pLagrangeMultiplier);
	return fast_atan(lagrangeSeries)*0.318309886f+0.5f;
}


